unit Sample1u;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs, StdCtrls,
  ExtCtrls, ComCtrls;

const
  AM_READDATA = WM_USER + 71;

type
  TSampleForm = class(TForm)
    LogMemo: TMemo;
    Panel1: TPanel;
    Label1: TLabel;
    BaseEdit: TEdit;
    Label2: TLabel;
    IRQEdit: TEdit;
    Label3: TLabel;
    Label4: TLabel;
    SecondsEdit: TEdit;
    FileEdit: TEdit;
    BeginBTN: TButton;
    procedure BeginBTNClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
  protected
    procedure AMReadData(var Msg: TMessage); message AM_READDATA;
  private
    { Private declarations }
  public
    { Public declarations }
  end;

var
  SampleForm: TSampleForm;
  channels: integer;
  dac0mult, dac1mult: double;
  bInitialized: Boolean = False;
  IRQNum: Integer;

implementation

{$R *.DFM}

uses
  ACCES32, Win32IRQ, AIO16A;

const
  FIFOHalfWords = 512;
  FIFOHalfBytes = 2 * FIFOHalfWords;
  NO_CARD = $FFFFFFFF;

type
  TAcqThread = class(TThread)
  public
    SamplesLeftToTake: Integer;
  protected
    procedure Execute; override;
  end;

var
  ADData: array of Word;
  AcqThread: TAcqThread = nil;
  LogFile: TFileStream;

procedure CtrMode(cntr, mode: Byte);
var
  ctrl: byte;

begin
  ctrl := (cntr shl 6) or $30 or (mode shl 1);
  OutPortB(Base+8+3, ctrl);
end;

procedure CtrLoad(c, val: Word);
begin
  OutPortB(Base+8+c, lo(val));
  OutPortB(Base+8+c, hi(val));
end;

{ TSampleForm }

procedure TSampleForm.FormCreate(Sender: TObject);
begin
  Base := 0;

  LogMemo.Text := 'Please enter the IRQ and base address (in hex) of the 104-AIO16x, and then click Begin to begin acquisition.'#13#10#13#10;
end;

procedure TSampleForm.BeginBTNClick(Sender: TObject);
var
  Size: Integer;
  bSucc: Boolean;

begin
    Base := StrToInt('$' + BaseEdit.Text);

    IRQNum := StrToInt('$' + IRQEdit.Text);

    if AcqThread <> nil then begin
      Beep;
      Exit;
    end;

    if not bInitialized then begin
      bSucc := InitGenDriver(Base, IRQNum, ISA, 0, $13, READ_TO_CLEAR);
      if not bSucc then LogMemo.Lines.Add(
        'Driver error during initialization; make sure base address ' + IntToHex(Base, 1) + 'h and IRQ ' + IntToHex(IRQNum, 1) + 'h aren''t used as PCI resources in Device Manager.'#13#10#13#10
      +
        'You may need to set IRQ ' + IntToHex(IRQNum, 1) + 'h to "legacy" or "reserved" in the BIOS.'#13#10#13#10
      );
      bInitialized := True;
    end;



    //Set buffer size based on seconds to acquire.
    Size := StrToInt(SecondsEdit.Text);
    SetLength(ADData, Size * 16000); //The counter load values below take 16k samples per second.

    //Open the file we'll save the data to.
    LogFile := TFileStream.Create(FileEdit.Text, fmCreate);

    //Reset entire board to clear out any previous config.
    OutPortB(Base+$1B, $FF);

    //Initialize the global variables and then load the card's calibration.
    init;
    doCAL;

    //tell the card to take one reading per channel using Scan Start
    OutPortB(Base+$11, $05);//Set ADC SCAN MODE and COUNTER START

    OutPortB(Base+$13, $04);

    //Both A/D counters in mode 2; they'll be loaded below.
    CtrMode(1, 2);
    CtrMode(2, 2);

    //determine if card is in single ended or differential mode
    if (jumpers and $2) = 2 then begin
        channels := 16;
        OutPortB(Base+$06, $F0); //scan all channels, zero to fifteen

        //Ctr 1 & 2 divisor of 10000 with 10MHz clock gives 1kHz tick.
        //Scanning 16 channels, that's 16k samples per second.
        CtrLoad(1, 2);
        CtrLoad(2, 5000);
    end
    else begin
        channels := 8;
        OutPortB(Base+$06, $70); //scan all channels, zero to seven

        //Ctr 1 & 2 divisor of 5000 with 10MHz clock gives 2kHz tick.
        //Scanning 8 channels, that's 16k samples per second.
        CtrLoad(1, 2);
        CtrLoad(2, 2500);
    end;

    //Launch the acquisition thread.
    AcqThread := TAcqThread.Create(True);
    AcqThread.SamplesLeftToTake := Length(ADData) * SizeOf(ADData[0]);
    AcqThread.Resume;

    LogMemo.Lines.Add('Acquiring...');
end;

procedure TSampleForm.AMReadData(var Msg: TMessage);
var
  Content, SampleStr: String;
  I: Integer;

begin
  if AcqThread = nil then begin
    Exit;
  end
  else if AcqThread.ReturnValue <> ERROR_SUCCESS then begin
    //LogMemo.Lines.Add('Error ' + IntToStr(AcqThread.ReturnValue) + ' from WaitForIRQData.');
    LogMemo.Lines.Add('DetectIRQ returned an error.'); 
  end
  else begin
    LogMemo.Lines.Add('Acquisition complete, saving...');
    SetString(Content, nil, Length(ADData) * 6);
    for I := 0 to High(ADData) do begin
      SampleStr := IntToHex(ADData[I], 4) + #13#10;
      Move(SampleStr[1], Content[1 + 6*I], 6);
    end;
    LogFile.Write(Content[1], Length(Content));
    FreeAndNil(LogFile);
    LogMemo.Lines.Add('Saving complete.');
    FreeAndNil(AcqThread);
  end;
end;

{ TAcqThread }

procedure TAcqThread.Execute;
var
  SamplesThisFIFO: Integer;
  pNextAD: PWord;

begin
  ReturnValue := ERROR_SUCCESS;
  pNextAD := @ADData[0];
  while SamplesLeftToTake > 0 do begin
    SamplesThisFIFO := SamplesLeftToTake;
    if SamplesThisFIFO > FIFOHalfWords then SamplesThisFIFO := FIFOHalfWords;

    if not DetectIRQ then begin
      ReturnValue := ERROR_OPERATION_ABORTED;
      Break;
    end;

    INSW(Base+0, SamplesThisFIFO, pNextAD^);

    //At this point, another thread could process the part we've taken, but we'll wait for acquisition to finish because string processing is slow.

    Inc(pNextAD, SamplesThisFIFO);
    Dec(SamplesLeftToTake, SamplesThisFIFO);
  end;

  //Stop the A/D counters by moding them.
  CtrMode(1, 2);
  CtrMode(2, 2);

  PostMessage(SampleForm.Handle, AM_READDATA, 0, 0);
end;

end.
